#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME=$(uci get ghostblog.@ghostblog[0].image_name 2>/dev/null)
}

do_install_detail() {
  local config=$(uci get ghostblog.@ghostblog[0].config_path 2>/dev/null)
  local port=$(uci get ghostblog.@ghostblog[0].port 2>/dev/null)
  local IMAGE_NAME=$(uci get ghostblog.@ghostblog[0].image_name 2>/dev/null)

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  [ -z "$config" ] && echo "config path is empty!" && exit 1
  [ -z "$port" ] && port=2368
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME=ghost

  rm -R /opt/docker2/compose/ghost-app
  mkdir /opt/docker2/compose/ghost-app
  touch /opt/docker2/compose/ghost-app/docker-compose.yml
  cat > /opt/docker2/compose/ghost-app/docker-compose.yml <<EOF
version: '3.1'
services:
  ghost:
    image: ${IMAGE_NAME}
    ports:
      - ${LAN_IP}:${port}:2368
    environment:
      database__client: mysql
      database__connection__host: db
      database__connection__user: root
      database__connection__password: ${GEN_PASS}
      database__connection__database: ghost
      url: http://${LAN_IP}:${port}
    restart: always
  db:
    image: mysql:8.0
    environment:
      MYSQL_ROOT_PASSWORD: ${GEN_PASS}
    restart: always
EOF

  docker-compose -f /opt/docker2/compose/ghost-app/docker-compose.yml up -d

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install Ghost blog"
  echo "      upgrade                Upgrade Ghost blog"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart Ghost blog"
  echo "      status                 Ghost blog status"
  echo "      port                   Ghost blog port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac
